rocm-sys 0.0.1

ROCm system bindings configured by release version
1
2
3
4
5
6
7
8
9
10
11
12
13
14
15
16
17
18
19
20
21
22
23
24
25
26
27
28
29
30
31
32
33
34
35
36
37
38
39
40
41
42
43
44
45
46
47
48
49
50
51
52
53
54
55
56
57
58
59
60
61
62
63
64
65
66
67
68
69
70
71
72
73
74
75
76
77
78
79
80
81
82
83
84
85
86
87
88
89
90
91
92
93
94
95
96
97
98
99
100
101
102
103
104
105
106
107
108
109
110
111
112
113
114
115
116
117
118
119
120
121
122
123
124
125
126
127
128
129
130
131
132
133
134
135
136
137
138
139
140
141
142
143
144
145
146
147
148
149
150
151
152
153
154
155
156
157
158
159
160
161
162
163
164
165
166
167
168
169
170
171
172
173
174
175
176
177
178
179
180
181
182
183
184
185
186
187
188
189
190
191
192
193
194
195
196
197
198
199
200
201
202
203
204
205
206
207
208
209
210
211
212
213
214
215
216
217
218
219
220
221
222
223
224
225
226
227
228
229
230
231
232
233
234
235
236
237
238
239
240
241
242
243
244
245
246
247
248
249
250
251
252
253
254
255
256
257
258
259
260
261
262
263
264
265
266
267
268
269
270
271
272
273
274
275
276
277
278
279
280
281
282
283
284
285
286
287
288
289
290
291
292
293
294
295
296
297
298
299
300
301
302
303
304
305
306
307
308
309
310
311
312
313
314
315
316
317
318
319
320
321
322
323
324
325
326
327
328
329
330
331
332
333
334
335
336
337
338
339
340
341
342
343
344
345
346
347
348
349
350
351
352
353
354
355
356
357
358
359
360
361
362
363
364
365
366
367
368
369
370
371
372
373
374
375
376
377
378
379
380
381
382
383
384
385
386
387
388
389
390
391
392
393
394
395
396
397
398
399
400
401
402
403
404
405
406
407
408
409
410
411
412
413
414
415
416
417
418
419
420
421
422
423
424
425
426
427
428
429
430
431
432
433
434
435
436
437
438
439
440
441
442
443
444
445
446
447
448
449
450
451
452
453
454
455
456
457
458
459
460
461
462
463
464
465
466
467
468
469
470
471
472
473
474
475
476
477
478
479
480
481
482
483
484
485
486
487
488
489
490
491
492
493
494
495
496
497
498
499
500
501
502
503
504
505
506
507
508
509
510
511
512
513
514
515
516
517
518
519
520
521
522
523
524
525
526
527
528
529
530
531
532
533
534
535
536
537
538
539
540
541
542
543
544
545
546
547
548
549
550
551
552
553
554
555
556
557
558
559
560
561
562
563
564
565
566
567
568
569
570
571
572
573
574
575
576
577
578
579
580
581
582
583
584
585
586
587
588
589
590
591
592
593
594
595
596
597
598
599
600
601
602
603
604
605
606
607
608
609
610
611
612
613
614
615
616
617
618
619
620
621
622
623
624
625
626
627
628
629
630
631
632
633
634
635
636
637
638
639
640
641
642
643
644
645
646
647
648
649
650
651
652
653
654
655
656
657
658
659
660
661
662
663
664
665
666
667
668
669
670
671
672
673
674
675
676
677
678
679
680
681
682
683
684
685
686
687
688
689
690
691
692
693
694
695
696
697
698
699
700
701
702
703
704
705
706
707
708
709
710
711
712
713
714
715
716
717
718
719
720
721
722
723
724
725
726
727
728
729
730
731
732
733
734
735
736
737
738
739
740
741
742
743
744
745
746
747
748
749
750
751
752
753
754
755
756
757
758
759
760
761
762
763
764
765
766
767
768
769
770
771
772
773
774
775
776
777
778
779
780
781
782
783
784
785
786
787
788
789
790
791
792
793
794
795
796
797
798
799
800
801
802
803
804
805
806
807
808
809
810
811
812
813
814
815
816
817
818
819
820
821
822
823
824
825
826
827
828
829
830
831
832
833
834
835
836
837
838
839
840
841
842
843
844
845
846
847
848
849
850
851
852
853
854
855
856
857
858
859
860
861
862
863
864
865
866
867
868
869
870
871
872
873
874
875
876
877
878
879
880
881
882
883
884
885
886
887
888
889
890
891
892
893
894
895
896
897
898
899
900
901
902
903
904
905
906
907
908
909
910
911
912
913
914
915
916
917
918
919
920
921
922
923
924
925
926
927
928
929
930
931
932
933
934
935
936
937
938
939
940
941
942
943
944
945
946
947
948
949
950
951
952
953
954
955
956
957
958
959
960
961
962
963
964
965
966
967
968
969
970
971
972
973
974
975
976
977
978
979
980
981
982
983
984
985
986
987
988
989
990
991
992
993
994
995
996
997
998
999
1000
1001
1002
1003
1004
1005
1006
1007
1008
1009
1010
1011
1012
1013
1014
1015
1016
1017
1018
1019
1020
1021
1022
1023
1024
//! All HIP/HIPRTC functions and types as of ROCm 7.15.

mod types;

pub use types::*;

use crate::{
    version::{ROCM_MAJOR as MAJOR, ROCM_MINOR as MINOR},
};
use core::ffi::*;

link! {
    amdhip64: [7, 8, 9, 10];

    pub fn hipInit(flags: c_uint) -> hipError_t;

    pub fn hipDriverGetVersion(driverVersion: *mut c_int) -> hipError_t;
    pub fn hipRuntimeGetVersion(runtimeVersion: *mut c_int) -> hipError_t;

    pub fn hipDeviceGet(
        device: *mut hipDevice_t,
        ordinal: c_int,
    ) -> hipError_t;
    pub fn hipDeviceComputeCapability(
        major: *mut c_int,
        minor: *mut c_int,
        device: hipDevice_t,
    ) -> hipError_t;
    pub fn hipDeviceGetName(
        name: *mut c_char,
        len: *mut c_int,
        device: hipDevice_t,
    ) -> hipError_t;

    #[since = 5 .1]
    pub fn hipDeviceGetUuid(
        uuid: *mut hipUUID,
        device: hipDevice_t,
    ) -> hipError_t;

    #[since = 5 .1]
    pub fn hipDeviceGetLuid(
        luid: *mut c_char,
        deviceNodeMask: *mut c_uint,
        device: hipDevice_t,
    ) -> hipError_t;

    pub fn hipDeviceGetP2PAttribute(
        value: *mut c_int,
        attr: hipDeviceP2PAttr,
        srcDevice: c_int,
        dstDevice: c_int,
    ) -> hipError_t;

    pub fn hipDeviceGetPCIBusId(
        pciBusId: *mut c_char,
        len: c_int,
        device: c_int,
    ) -> hipError_t;

    pub fn hipDeviceGetByPCIBusId(
        device: *mut c_int,
        pciBusId: *const c_char
    ) -> hipError_t;

    pub fn hipDeviceTotalMem(
        bytes: *mut usize,
        device: hipDevice_t,
    ) -> hipError_t;

    pub fn hipDeviceSynchronize() -> hipError_t;
    pub fn hipDeviceReset() -> hipError_t;

    pub fn hipSetDevice(deviceId: c_int) -> hipError_t;
    pub fn hipSetValidDevices(
        device_arr: *mut c_int,
        len: c_int,
    ) -> hipError_t;

    pub fn hipGetDevice(device: *mut c_int) -> hipError_t;
    pub fn hipGetDeviceCount(count: *mut c_int) -> hipError_t;

    pub fn hipDeviceGetAttribute(
        pi: *mut c_int,
        attr: hipDeviceAttribute_t,
        deviceId: c_int,
    ) -> hipError_t;

    #[since = 5 .1]
    pub fn hipDeviceGetDefaultMemPool(
        mem_pool: *mut hipMemPool_t,
        device: c_int,
    ) -> hipError_t;

    #[since = 5 .1]
    pub fn hipDeviceSetMemPool(
        device: c_int,
        mem_pool: hipMemPool_t,
    ) -> hipError_t;

    #[since = 5 .1]
    pub fn hipDeviceGetMemPool(
        device: c_int,
        mem_pool: *mut hipMemPool_t,
    ) -> hipError_t;

    #[doc = "assumes hipDeviceProp_tR0600=hipDeviceProp_t"]
    pub fn hipGetDeviceProperties(
        prop: *mut hipDeviceProp_tR0600,
        deviceId: c_int,
    ) -> hipError_t;

    pub fn hipDeviceSetCacheConfig(
        cacheConfig: hipFuncCache_t,
    ) -> hipError_t;
    pub fn hipDeviceGetCacheConfig(
        cacheConfig: *mut hipFuncCache_t,
    ) -> hipError_t;

    pub fn hipDeviceGetLimit(
        pValue: *mut usize,
        limit: hipLimit_t,
    ) -> hipError_t;

    #[since = 5 .3]
    pub fn hipDeviceSetLimit(
        limit: *mut hipLimit_t,
        pValue: usize,
    ) -> hipError_t;

    pub fn hipDeviceGetSharedMemConfig(
        pConfig: *mut hipSharedMemConfig,
    ) -> hipError_t;

    pub fn hipGetDeviceFlags(
        flags: *mut c_uint,
    ) -> hipError_t;

    pub fn hipDeviceSetSharedMemConfig(
        config: hipSharedMemConfig,
    ) -> hipError_t;

    pub fn hipSetDeviceFlags(
        flags: c_uint,
    ) -> hipError_t;

    pub fn hipInitDevice(
        device: c_int,
        deviceFlags: c_uint,
        flags: c_uint,
    ) -> hipError_t;

    #[doc = "assumes hipDeviceProp_tR0600=hipDeviceProp_t"]
    pub fn hipChooseDevice(
        device: *mut c_int,
        prop: *const hipDeviceProp_tR0600,
    ) -> hipError_t;

    pub fn hipExtGetLinkTypeAndHopCount(
        device1: c_int,
        device2: c_int,
        linktype: *mut u32,
        hopcount: u32,
    ) -> hipError_t;

    pub fn hipIpcGetMemHandle(
        handle: *mut hipIpcMemHandle_t,
        devPtr: *mut c_void,
    ) -> hipError_t;

    pub fn hipIpcOpenMemHandle(
        devPtr: *mut *mut c_void,
        handle: hipIpcMemHandle_t,
        flags: c_uint,
    ) -> hipError_t;

    pub fn hipIpcCloseMemHandle(
        devPtr: *mut c_void,
    ) -> hipError_t;

    pub fn hipIpcGetEventHandle(
        handle: *mut hipIpcEventHandle_t,
        event: hipEvent_t,
    ) -> hipError_t;

    pub fn hipIpcOpenEventHandle(
        event: *mut hipEvent_t,
        handle: hipIpcEventHandle_t,
    ) -> hipError_t;

    pub fn hipExtModuleLaunchKernel(
        f: hipFunction_t,
        globalWorkSizeX: u32,
        globalWorkSizeY: u32,
        globalWorkSizeZ: u32,
        localWorkSizeX: u32,
        localWorkSizeY: u32,
        localWorkSizeZ: u32,
        sharedMemBytes: usize,
        hStream: hipStream_t,
        kernelParams: *mut *mut c_void,
        extra: *mut *mut c_void,
        startEvent: hipEvent_t,
        stopEvent: hipEvent_t,
        flags: u32,
    ) -> hipError_t;

    pub fn hipHccModuleLaunchKernel(
        f: hipFunction_t,
        globalWorkSizeX: u32,
        globalWorkSizeY: u32,
        globalWorkSizeZ: u32,
        localWorkSizeX: u32,
        localWorkSizeY: u32,
        localWorkSizeZ: u32,
        sharedMemBytes: usize,
        hStream: hipStream_t,
        kernelParams: *mut *mut c_void,
        extra: *mut *mut c_void,
        startEvent: hipEvent_t,
        stopEvent: hipEvent_t,
    ) -> hipError_t;

    pub fn hipFuncSetAttribute(
        func: *const c_void,
        attr: hipFuncAttribute,
        value: c_int,
    ) -> hipError_t;

    #[since = 7 .2]
    pub fn hipKernelSetAttribute(
        attrib: hipFunction_attribute,
        value: c_int,
        kernel: hipKernel_t,
        dev: hipDevice_t,
    ) -> hipError_t;

    #[since = 7 .2]
    pub fn hipKernelGetFunction(
        pFunc: *mut hipFunction_t,
        kernel: hipKernel_t,
    ) -> hipError_t;

    pub fn hipFuncSetCacheConfig(
        func: *const c_void,
        config: hipFuncCache_t,
    ) -> hipError_t;

    pub fn hipFuncSetSharedMemConfig(
        func: *const c_void,
        config: hipSharedMemConfig,
    ) -> hipError_t;

    pub fn hipFuncGetAttributes(
        attr: *mut hipFuncAttributes,
        func: *const c_void,
    ) -> hipError_t;

    pub fn hipFuncGetAttribute(
        value: *mut c_int,
        attrib: hipFunction_attribute,
        hfunc: hipFunction_t,
    ) -> hipError_t;

    pub fn hipModuleLaunchKernel(
        f: hipFunction_t,
        gridDimX: c_uint,
        gridDimY: c_uint,
        gridDimZ: c_uint,
        blockDimX: c_uint,
        blockDimY: c_uint,
        blockDimZ: c_uint,
        sharedMemBytes: c_uint,
        stream: hipStream_t,
        kernelParams: *mut *mut c_void,
        extra: *mut *mut c_void,
    ) -> hipError_t;

    pub fn hipExtLaunchMultiKernelMultiDevice(
        launchParamsList: *mut hipLaunchParams,
        numDevices: c_int,
        flags: c_uint,
    ) -> hipError_t;

    #[since = 6 .5]
    pub fn hipLaunchKernelExC(
        config: *const hipLaunchConfig_t,
        fPtr: *const c_void,
        args: *mut *mut c_void,
    ) -> hipError_t;

    #[since = 6 .5]
    pub fn hipDrvLaunchKernelEx(
        config: *const HIP_LAUNCH_CONFIG,
        f: hipFunction_t,
        params: *mut *mut c_void,
        extra: *mut *mut c_void,
    ) -> hipError_t;

    pub fn hipGetLastError() -> hipError_t;

    #[since = 6 .0]
    pub fn hipExtGetLastError() -> hipError_t;

    pub fn hipGetErrorName(hip_error: hipError_t) -> *const c_char;
    pub fn hipGetErrorString(hip_error: hipError_t) -> *const c_char;

    #[since = 5 .3]
    pub fn hipDrvGetErrorName(
        hip_Error: hipError_t,
        errorString: *mut *const c_char,
    ) -> hipError_t;

    #[since = 5 .3]
    pub fn hipDrvGetErrorString(
        hip_Error: hipError_t,
        errorString: *mut *const c_char,
    ) -> hipError_t;

    #[since = 7 .14]
    pub fn hipStreamCreate(
        stream: *mut hipStream_t,
    ) -> hipError_t;

    pub fn hipStreamCreateWithFlags(
        stream: *mut hipStream_t,
        flags: c_uint,
    ) -> hipError_t;
    pub fn hipStreamCreateWithPriority(
        stream: *mut hipStream_t,
        flags: c_uint,
        priority: c_int,
    ) -> hipError_t;

    pub fn hipDeviceGetStreamPriorityRange(
        leastPriority: *mut c_int,
        greatestPriority: *mut c_int,
    ) -> hipError_t;

    pub fn hipStreamDestroy(stream: hipStream_t) -> hipError_t;
    pub fn hipStreamQuery(stream: hipStream_t) -> hipError_t;
    pub fn hipStreamSynchronize(stream: hipStream_t) -> hipError_t;

    pub fn hipStreamWaitEvent(
        stream: hipStream_t,
        event: hipEvent_t,
        flags: c_uint,
    ) -> hipError_t;

    #[since = 5 .2]
    pub fn hipStreamGetFlags(
        stream: hipStream_t,
        flags: *mut c_uint,
    ) -> hipError_t;

    #[since = 7 .1]
    pub fn hipStreamGetId(
        stream: hipStream_t,
        streamId: *mut c_ulonglong,
    ) -> hipError_t;

    pub fn hipStreamGetPriority(
        stream: hipStream_t,
        priority: *mut c_int,
    ) -> hipError_t;

    pub fn hipStreamGetDevice(
        stream: hipStream_t,
        device: *mut hipDevice_t,
    ) -> hipError_t;

    pub fn hipExtStreamCreateWithCUMask(
        stream: *mut hipStream_t,
        cuMaskSize: u32,
        cuMask: *const u32,
    ) -> hipError_t;

    pub fn hipExtStreamGetCUMask(
        stream: hipStream_t,
        cuMaskSize: u32,
        cuMask: *mut u32,
    ) -> hipError_t;

    pub fn hipStreamAddCallback(
        stream: hipStream_t,
        callback: hipStreamCallback_t,
        userData: *mut c_void,
        flags: c_uint,
    ) -> hipError_t;

    #[since = 7 .1]
    pub fn hipStreamSetAttribute(
        stream: hipStream_t,
        attr: hipLaunchAttributeID,
        value: *const hipLaunchAttributeValue,
    ) -> hipError_t;

    #[since = 7 .1]
    pub fn hipStreamGetAttribute(
        stream: hipStream_t,
        attr: hipLaunchAttributeID,
        value_out: *mut hipLaunchAttributeValue,
    ) -> hipError_t;

    #[since = 7 .2]
    pub fn hipStreamCopyAttributes(
        dst: hipStream_t,
        src: hipStream_t,
    ) -> hipError_t;

    #[since = 4 .4]
    pub fn hipStreamWaitValue32(
        stream: hipStream_t,
        ptr: *mut c_void,
        value: u32,
        flags: c_uint,
        mask: u32,
    ) -> hipError_t;

    #[since = 4 .4]
    pub fn hipStreamWaitValue64(
        stream: hipStream_t,
        ptr: *mut c_void,
        value: u64,
        flags: c_uint,
        mask: u64,
    ) -> hipError_t;

    #[since = 4 .4]
    pub fn hipStreamWriteValue32(
        stream: hipStream_t,
        ptr: *mut c_void,
        value: u32,
        flags: c_uint,
    ) -> hipError_t;

    #[since = 4 .4]
    pub fn hipStreamWriteValue64(
        stream: hipStream_t,
        ptr: *mut c_void,
        value: u64,
        flags: c_int,
    ) -> hipError_t;

    #[since = 6 .4]
    pub fn hipStreamBatchMemOp(
        stream: hipStream_t,
        count: c_int,
        paramArray: *mut hipStreamBatchMemOpParams,
        flags: c_int,
    ) -> hipError_t;

    #[since = 6 .4]
    pub fn hipGraphAddBatchMemOpNode(
        phGraphNode: *mut hipGraphNode_t,
        hGraph: hipGraph_t,
        dependencies: *const hipGraphNode_t,
        numDependencies: usize,
        nodeParams: *const hipBatchMemOpNodeParams,
    ) -> hipError_t;

    #[since = 6 .4]
    pub fn hipGraphBatchMemOpNodeGetParams(
        hNode: hipGraphNode_t,
        nodeParams_out: *mut hipBatchMemOpNodeParams,
    ) -> hipError_t;

    #[since = 6 .4]
    pub fn hipGraphBatchMemOpNodeSetParams(
        hNode: hipGraphNode_t,
        nodeParams: *mut hipBatchMemOpNodeParams,
    ) -> hipError_t;

    #[since = 6 .4]
    pub fn hipGraphExecBatchMemOpNodeSetParams(
        hGraphExec: hipGraphExec_t,
        hNode: hipGraphNode_t,
        nodeParams: *const hipBatchMemOpNodeParams,
    ) -> hipError_t;

    pub fn hipEventCreateWithFlags(
        event: *mut hipEvent_t,
        flags: c_uint,
    ) -> hipError_t;

    pub fn hipEventCreate(event: *mut hipEvent_t) -> hipError_t;

    #[since = 6 .4]
    pub fn hipEventRecordWithFlags(
        event: hipEvent_t,
        stream: hipStream_t,
        flags: c_uint,
    ) -> hipError_t;

    pub fn hipEventRecord(
        event: hipEvent_t,
        stream: hipStream_t,
    ) -> hipError_t;
    pub fn hipEventDestroy(event: hipEvent_t) -> hipError_t;

    pub fn hipEventSynchronize(event: hipEvent_t) -> hipError_t;

    pub fn hipEventElapsedTime(
        ms: *mut f32,
        start: hipEvent_t,
        stop: hipEvent_t,
    ) -> hipError_t;

    pub fn hipEventQuery(event: hipEvent_t) -> hipError_t;

    pub fn hipPointerSetAttribute(
        value: *const c_void,
        attribute: hipPointer_attribute,
        ptr: hipDeviceptr_t,
    ) -> hipError_t;

    pub fn hipPointerGetAttributes(
        attributes: *mut hipPointerAttribute_t,
        ptr: *const c_void,
    ) -> hipError_t;
    pub fn hipPointerGetAttribute(
        data: *mut c_void,
        attribute: hipPointer_attribute,
        ptr: hipDeviceptr_t,
    ) -> hipError_t;

    #[since = 5 .0]
    pub fn hipDrvPointerGetAttributes(
        numAttributes: c_uint,
        attributes: *mut hipPointer_attribute,
        data: *mut *mut c_void,
        ptr: hipDeviceptr_t,
    ) -> hipError_t;

    pub fn hipMalloc(
        ptr: *mut *mut c_void,
        size: usize,
    ) -> hipError_t;
    pub fn hipExtMallocWithFlags(
        ptr: *mut *mut c_void,
        sizeBytes: usize,
        flags: c_uint,
    ) -> hipError_t;

    pub fn hipHostMalloc(
        ptr: *mut *mut c_void,
        size: usize,
        flags: c_uint,
    ) -> hipError_t;
    pub fn hipHostAlloc(
        ptr: *mut *mut c_void,
        size: usize,
        flags: c_uint,
    ) -> hipError_t;

    pub fn hipHostGetDevicePointer(
        devPtr: *mut *mut c_void,
        hstPtr: *mut c_void,
        flags: c_uint,
    ) -> hipError_t;
    pub fn hipHostGetFlags(
        flagsPtr: * mut c_uint,
        hostPtr: *mut c_void,
    ) -> hipError_t;

    pub fn hipHostRegister(
        hostPtr: *mut c_void,
        sizeBytes: usize,
        flags: c_uint,
    ) -> hipError_t;

    pub fn hipHostUnregister(hostPtr: *mut c_void) -> hipError_t;

    pub fn hipMallocPitch(
        ptr: *mut *mut c_void,
        pitch: *mut usize,
        width: usize,
        height: usize,
    ) -> hipError_t;

    pub fn hipMemAllocPitch(
        dptr: *mut hipDeviceptr_t,
        pitch: *mut usize,
        widthInBytes: usize,
        height: usize,
        elementSizeBytes: c_uint,
    ) -> hipError_t;

    pub fn hipFree(
        ptr: *mut c_void,
    ) -> hipError_t;
    pub fn hipFreeHost(
        ptr: *mut c_void,
    ) -> hipError_t;

    pub fn hipMemcpy(
        dst: *mut c_void,
        src: *const c_void,
        size: usize,
        kind: hipMemcpyKind,
    ) -> hipError_t;
    pub fn hipMemcpyWithStream(
        dst: *mut c_void,
        src: *const c_void,
        sizeBytes: usize,
        kind: hipMemcpyKind,
        stream: hipStream_t,
    ) -> hipError_t;

    pub fn hipMemcpyHtoD(
        dst: hipDeviceptr_t,
        src: *const c_void,
        sizeBytes: usize,
    ) -> hipError_t;
    pub fn hipMemcpyDtoH(
        dst: *mut c_void,
        src: hipDeviceptr_t,
        sizeBytes: usize,
    ) -> hipError_t;
    pub fn hipMemcpyDtoD(
        dst: hipDeviceptr_t,
        src: hipDeviceptr_t,
        sizeBytes: usize,
    ) -> hipError_t;

    #[since = 6 .2]
    pub fn hipMemcpyAtoD(
        dstDevice: hipDeviceptr_t,
        srcArray: hipArray_t,
        srcOffset: usize,
        ByteCount: usize,
    ) -> hipError_t;

    #[since = 6 .2]
    pub fn hipMemcpyDtoA(
        dstArray: hipArray_t,
        dstOffset: usize,
        srcDevice: hipDeviceptr_t,
        ByteCount: usize,
    ) -> hipError_t;

    #[since = 6 .2]
    pub fn hipMemcpyAtoA(
        dstArray: hipArray_t,
        dstOffset: usize,
        srcArray: hipArray_t,
        srcOffset: usize,
        ByteCount: usize,
    ) -> hipError_t;

    pub fn hipMemcpyHtoDAsync(
        dst: hipDeviceptr_t,
        src: *const c_void,
        sizeBytes: usize,
        stream: hipStream_t,
    ) -> hipError_t;
    pub fn hipMemcpyDtoHAsync(
        dst: *mut c_void,
        src: hipDeviceptr_t,
        sizeBytes: usize,
        stream: hipStream_t,
    ) -> hipError_t;
    pub fn hipMemcpyDtoDAsync(
        dst: hipDeviceptr_t,
        src: hipDeviceptr_t,
        sizeBytes: usize,
        stream: hipStream_t,
    ) -> hipError_t;

    #[since = 6 .2]
    pub fn hipMemcpyAtoHAsync(
        dstHost: *mut c_void,
        srcArray: hipArray_t,
        srcOffset: usize,
        ByteCount: usize,
        stream: hipStream_t,
    ) -> hipError_t;

    #[since = 6 .2]
    pub fn hipMemcpyHtoAAsync(
        dstArray: hipArray_t,
        dstOffset: usize,
        srcHost: *const c_void,
        ByteCount: usize,
        stream: hipStream_t,
    ) -> hipError_t;

    pub fn hipGetSymbolAddress(
        devPtr: *mut *mut c_void,
        symbol: *const c_void,
    ) -> hipError_t;
    pub fn hipGetSymbolSize(
        size: *mut usize,
        symbol: *const c_void,
    ) -> hipError_t;

    #[since = 6 .1]
    pub fn hipGetProcAddress(
        symbol: *const char,
        pfn: *mut *mut c_void,
        hipVersion: c_int,
        flags: u64,
        symbolStatus: hipDriverProcAddressQueryResult,
    ) -> hipError_t;

    pub fn hipMemcpyToSymbol(
        symbol: *const c_void,
        src: *const c_void,
        sizeBytes: usize,
        offset: usize,
        kind: hipMemcpyKind,
    ) -> hipError_t;

    pub fn hipMemcpyToSymbolAsync(
        symbol: *const c_void,
        src: *const c_void,
        sizeBytes: usize,
        offset: usize,
        kind: hipMemcpyKind,
        stream: hipStream_t,
    ) -> hipError_t;

    pub fn hipMemcpyFromSymbol(
        dst: *mut c_void,
        symbol: *const c_void,
        sizeBytes: usize,
        offset: usize,
        kind: hipMemcpyKind,
    ) -> hipError_t;
    pub fn hipMemcpyFromSymbolAsync(
        dst: *mut c_void,
        symbol: *const c_void,
        sizeBytes: usize,
        offset: usize,
        kind: hipMemcpyKind,
        stream: hipStream_t,
    ) -> hipError_t;

    pub fn hipMemcpyAsync(
        dst: *mut c_void,
        src: *const c_void,
        sizeBytes: usize,
        kind: hipMemcpyKind,
        stream: hipStream_t,
    ) -> hipError_t;

    pub fn hipMemset(
        dst: *mut c_void,
        value: c_int,
        sizeBytes: usize,
    ) -> hipError_t;

    pub fn hipMemsetD8(
        dest: hipDeviceptr_t,
        value: c_uchar,
        count: usize,
    ) -> hipError_t;
    pub fn hipMemsetD8Async(
        dest: hipDeviceptr_t,
        value: c_uchar,
        count: usize,
        stream: hipStream_t,
    ) -> hipError_t;

    pub fn hipMemsetD16(
        dest: hipDeviceptr_t,
        value: c_ushort,
        count: usize,
    ) -> hipError_t;
    pub fn hipMemsetD16Async(
        dest: hipDeviceptr_t,
        value: c_ushort,
        count: usize,
        stream: hipStream_t,
    ) -> hipError_t;

    pub fn hipMemsetD32(
        dest: hipDeviceptr_t,
        value: c_int,
        count: usize,
    ) -> hipError_t;
    pub fn hipMemsetD32Async(
        dest: hipDeviceptr_t,
        value: c_int,
        count: usize,
    ) -> hipError_t;

    pub fn hipMemset2D(
        dst: *mut c_void,
        pitch: usize,
        value: c_int,
        width: usize,
        height: usize,
    ) -> hipError_t;
    pub fn hipMemset2DAsync(
        dst: *mut c_void,
        pitch: usize,
        value: c_int,
        width: usize,
        height: usize,
        stream: hipStream_t,
    ) -> hipError_t;

    pub fn hipMemset3D(
        pitchedDevPtr: hipPitchedPtr,
        value: c_int,
        extent: hipExtent,
    ) -> hipError_t;
    pub fn hipMemset3DAsync(
        pitchedDevPtr: hipPitchedPtr,
        value: c_int,
        extent: hipExtent,
        stream: hipStream_t,
    ) -> hipError_t;

    #[since = 7 .1]
    pub fn hipMemsetD2D8(
        dst: hipDeviceptr_t,
        dstPitch: usize,
        value: c_uchar,
        width: usize,
        height: usize,
    ) -> hipError_t;
    
    #[since = 7 .1]
    pub fn hipMemsetD2D8Async(
        dst: hipDeviceptr_t,
        dstPitch: usize,
        value: c_uchar,
        width: usize,
        height: usize,
        stream: hipStream_t,
    ) -> hipError_t;

    #[since = 7 .1]
    pub fn hipMemsetD2D16(
        dst: hipDeviceptr_t,
        dstPitch: usize,
        value: c_ushort,
        width: usize,
        height: usize,
    ) -> hipError_t;
    
    #[since = 7 .1]
    pub fn hipMemsetD2D16Async(
        dst: hipDeviceptr_t,
        dstPitch: usize,
        value: c_ushort,
        width: usize,
        height: usize,
        stream: hipStream_t,
    ) -> hipError_t;

    #[since = 7 .1]
    pub fn hipMemsetD2D32(
        dst: hipDeviceptr_t,
        dstPitch: usize,
        value: c_uint,
        width: usize,
        height: usize,
    ) -> hipError_t;

    #[since = 7 .1]
    pub fn hipMemsetD2D32Async(
        dst: hipDeviceptr_t,
        dstPitch: usize,
        value: c_uint,
        width: usize,
        height: usize,
        stream: hipStream_t,
    ) -> hipError_t;

    pub fn hipMemGetInfo(
        free: *mut usize,
        total: *mut usize,
    ) -> hipError_t;

    pub fn hipMemPtrGetInfo(
        ptr: *mut c_void,
        size: *mut usize,
    ) -> hipError_t;

    pub fn hipMallocArray(
        array: *mut hipArray_t,
        desc: *const hipChannelFormatDesc,
        width: usize,
        height: usize,
        flags: c_uint,
    ) -> hipError_t;

    pub fn hipArrayCreate(
        pHandle: *mut hipArray_t,
        pAllocateArray: *const HIP_ARRAY_DESCRIPTOR,
    ) -> hipError_t;
    pub fn hipArrayDestroy(array: hipArray_t) -> hipError_t;

    pub fn hipArray3DCreate(
        array: *mut hipArray_t,
        pAllocateArray: *const HIP_ARRAY3D_DESCRIPTOR,
    ) -> hipError_t;

    pub fn hipMalloc3D(
        pitchedDevPtr: *const hipPitchedPtr,
        extent: hipExtent,
    ) -> hipError_t;

    pub fn hipFreeArray(array: hipArray_t) -> hipError_t;

    pub fn hipMalloc3DArray(
        array: *mut hipArray_t,
        desc: *const hipChannelFormatDesc,
        extent: hipExtent,
        flags: c_uint,
    ) -> hipError_t;

    #[since = 5 .6]
    pub fn hipArrayGetInfo(
        desc: *mut hipChannelFormatDesc,
        extent: *mut hipExtent,
        flags: *mut c_uint,
        array: hipArray_t,
    ) -> hipError_t;

    #[since = 5 .6]
    pub fn hipArrayGetDescriptor(
        pArrayDescriptor: *mut HIP_ARRAY_DESCRIPTOR,
        array: hipArray_t,
    ) -> hipError_t;

    #[since = 5 .6]
    pub fn hipArray3DGetDescriptor(
        pArrayDescriptor: *mut HIP_ARRAY3D_DESCRIPTOR,
        array: hipArray_t,
    ) -> hipError_t;

    pub fn hipMemcpy2D(
        dst: *mut c_void,
        dpitch: usize,
        src: *const c_void,
        spitch: usize,
        width: usize,
        height: usize,
        kind: hipMemcpyKind,
    ) -> hipError_t;

    pub fn hipMemcpyParam2D(pCopy: *const hip_Memcpy2D) -> hipError_t;
    pub fn hipMemcpyParam2DAsync(
        pCopy: *const hip_Memcpy2D,
        stream: hipStream_t,
    ) -> hipError_t;

    pub fn hipMemcpy2DAsync(
        dst: *mut c_void,
        dpitch: usize,
        src: *const c_void,
        spitch: usize,
        width: usize,
        height: usize,
        kind: hipMemcpyKind,
        stream: hipStream_t,
    ) -> hipError_t;

    pub fn hipMemcpy2DToArray(
        dst: hipArray_t,
        wOffset: usize,
        hOffset: usize,
        src: *const c_void,
        spitch: usize,
        width: usize,
        height: usize,
        kind: hipMemcpyKind,
    ) -> hipError_t;

    #[since = 4 .3]
    pub fn hipMemcpy2DToArrayAsync(
        dst: hipArray_t,
        wOffset: usize,
        hOffset: usize,
        src: *const c_void,
        spitch: usize,
        width: usize,
        height: usize,
        kind: hipMemcpyKind,
        stream: hipStream_t,
    ) -> hipError_t;

    #[since = 6 .2]
    pub fn hipMemcpy2DArrayToArray(
        dst: hipArray_t,
        wOffsetDst: usize,
        hOffsetDst: usize,
        src: hipArray_const_t,
        wOffsetSrc: usize,
        hOffsetSrc: usize,
        width: usize,
        height: usize,
        kind: hipMemcpyKind,
    ) -> hipError_t;

    pub fn hipMemcpy2DFromArray(
        dst: *mut c_void,
        dpitch: usize,
        src: hipArray_const_t,
        wOffset: usize,
        hOffset: usize,
        width: usize,
        height: usize,
        kind: hipMemcpyKind,
    ) -> hipError_t;

    pub fn hipMemcpy2DFromArrayAsync(
        dst: *mut c_void,
        dpitch: usize,
        src: hipArray_const_t,
        wOffset: usize,
        hOffset: usize,
        width: usize,
        height: usize,
        kind: hipMemcpyKind,
        stream: hipStream_t,
    ) -> hipError_t;

    pub fn hipGraphicsUnregisterResource(resource: hipGraphicsResource_t) -> hipError_t;
}